import hashlib
import os
import pickle
import threading
import time
from typing import Union

import feedparser
import wx
from Channel import Channel
from Items import Item
from Utils import get_default_logo, make_bitmap_from_url
from youtube_dl import YoutubeDL as yt
from youtube_dl.utils import DownloadError, ExtractorError


class YouTube(Channel):
    m_cache: dict = dict()

    def __init__(self, channel_id: str, name: str) -> None:
        self.m_channel_id = channel_id
        self.m_name = name
        rss_url = 'https://www.youtube.com/feeds/videos.xml?channel_id={}'.format(
            channel_id)
        self.m_cachefile = '/tmp/yt_cache_{}'.format(channel_id)
        self.m_logo = get_default_logo('YouTube')
        super().__init__(channel_id, rss_url, self.m_logo,
                         self.m_name)
        self.m_items: Union[list[Item], None] = None

        if os.path.exists(self.m_cachefile):
            with open(self.m_cachefile, 'rb') as cachehandle:
                try:
                    self.m_cache = pickle.load(cachehandle)
                except EOFError or pickle.UnpicklingError:
                    pass

        self.m_thr = threading.Thread(target=self.parse_feed,
                                      args=(),
                                      kwargs={})
        self.m_thr.start()

        self.pickle()

    def wait(self) -> bool:
        return self.m_thr.is_alive()

    def parse_feed(self) -> None:
        feed = feedparser.parse(self.get_feed())

        ydl_opts = {
            'format':
            'worstvideo[ext=mp4]+worstaudio[ext=m4a]/worstvideo+worstaudio',
            'container': 'webm_dash',
        }
        entries = feed['entries']
        self.m_items: list[Item] = list()

        for entry in entries:
            key = hashlib.sha256(entry['link'].encode('utf-8')).hexdigest()

            if key in self.m_cache.keys():
                thumbnail_link = self.m_cache[key]['thumbnail_link']
                resolved_link = self.m_cache[key]['resolved_link']
                description = self.m_cache[key]['description']
                published_parsed = self.m_cache[key]['published_parsed']
                title = self.m_cache[key]['title']
            else:

                title = str(entry['title'])
                thumbnail_link = str(entry['media_thumbnail'][0]['url'])
                description = str(entry['description'])
                link = ''
                with yt(ydl_opts) as ydl:
                    try:
                        video = ydl.extract_info(entry['link'], download=False)

                        for form in video['formats']:
                            if form['height']:
                                if form['height'] < 480 and form[
                                        'acodec'] != 'none':
                                    link = form['url']
                    except ExtractorError or DownloadError:
                        pass

                resolved_link = link

                published_parsed = entry['published_parsed']

            if not resolved_link:
                continue
            self.m_cache[key] = {'thumbnail_link': thumbnail_link}
            self.m_cache[key]['resolved_link'] = resolved_link
            self.m_cache[key]['description'] = description
            self.m_cache[key]['published_parsed'] = published_parsed
            self.m_cache[key]['title'] = title
            thumbnail = make_bitmap_from_url(thumbnail_link,
                                             wx.Size(self.m_screen_width, 150))
            item = Item(description, resolved_link, self.m_provider_name,
                        published_parsed, thumbnail, title)
            self.m_items.append(item)

    def pickle(self) -> None:
        while self.wait():
            time.sleep(1)

        # write to cache file
        with open(self.m_cachefile, 'wb') as cachehandle:
            pickle.dump(self.m_cache, cachehandle)
